#import "AppDelegate.h"
#import "CallViewController.h"
#import "CallViewController+UI.h"
#import "SINCallKitMediator.h"
#import <Sinch/SINUIView+Fullscreen.h>

// Usage of gesture recognizers to toggle between front- and back-camera and fullscreen.
//
// * Triple tap the local preview view to pause sending local video stream to remote.
// * Double tap the local preview view to switch between front- and back-camera.
// * Single tap the local video stream view to make it go full screen.
//     Tap again to go back to normal size.
// * Single tap the remote video stream view to make it go full screen.
//     Tap again to go back to normal size.

@interface CallViewController () <SINCallKitMediatorObserver>
@property (weak, nonatomic) IBOutlet UIGestureRecognizer *remoteVideoFullscreenGestureRecognizer;
@property (weak, nonatomic) IBOutlet UIGestureRecognizer *localVideoFullscreenGestureRecognizer;
@property (weak, nonatomic) IBOutlet UIGestureRecognizer *switchCameraGestureRecognizer;
@property (weak, nonatomic) IBOutlet UITapGestureRecognizer *pauseResumeVideoRecognizer;
@property (nonatomic, readwrite) BOOL videoPaused;
@property (nonatomic, strong, readonly) id<SINClient> client;
@end

@implementation CallViewController

- (id<SINClient>)client {
  return self.callKitMediator.client;
}

- (void)setCallKitMediator:(SINCallKitMediator *)callKitMediator {
  _callKitMediator = callKitMediator;
  [_callKitMediator addObserver:self];
}

#pragma mark - UIViewController Cycle

- (void)viewDidLoad {
  [super viewDidLoad];

  NSAssert(self.callKitMediator != nil, @"self.callKitMediator must not be nil here", @"");
  NSAssert(self.callKitMediator.client != nil, @"self.callKitMediator.client must not be nil here", @"");

  [self setupForCallWithDirection:self.call.direction];

  if ([self.call.details isVideoOffered]) {
    [self.localVideoView addSubview:[self.client.videoController localView]];
    [self.localVideoFullscreenGestureRecognizer requireGestureRecognizerToFail:self.switchCameraGestureRecognizer];
    [self.localVideoFullscreenGestureRecognizer requireGestureRecognizerToFail:self.pauseResumeVideoRecognizer];
    [self.switchCameraGestureRecognizer requireGestureRecognizerToFail:self.pauseResumeVideoRecognizer];
    [[[self.client videoController] localView] addGestureRecognizer:self.localVideoFullscreenGestureRecognizer];
    [[[self.client videoController] localView] addGestureRecognizer:self.pauseResumeVideoRecognizer];
    [[[self.client videoController] remoteView] addGestureRecognizer:self.remoteVideoFullscreenGestureRecognizer];
    _videoPaused = NO;
  }
}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  self.remoteUsername.text = [self.call remoteUserId];
  [[self.client audioController] enableSpeaker];
}

- (void)setupForCallWithDirection:(SINCallDirection)direction {
  if (direction == SINCallDirectionIncoming) {
    // Enter the view from CallKit incoming call
    if ([self.callKitMediator callExistsWithId:self.call.callId]) {
      // Enter from CallKit lock screen
      if ([self.call state] == SINCallStateEstablished) {
        [self startCallDurationTimerWithSelector:@selector(onDurationTimer:)];
        [self showButtons:kButtonsHangup];
        [self showRemoteVideoView];
      } else {
        // Enter from CallKit but not lock screen
        [self setCallStatusText:@""];
        [self showButtons:kButtonsWakenByCallKit];
        [self showRemoteVideoView];
      }
    } else {
      // Enter the view from normal incoming call
      NSAssert(NO, @"Setup UI for normal incoming call not possible with CallKit. Call should exist in registry.", @"");
    }
  } else {  // Outgoing
    [self setCallStatusText:@"calling..."];
    [self showButtons:kButtonsHangup];
  }
}

#pragma mark - Call Actions

- (IBAction)hangup:(id)sender {
  [self.callKitMediator endCall:self.call];
}

- (IBAction)onSwitchCameraTapped:(id)sender {
  AVCaptureDevicePosition current = self.client.videoController.captureDevicePosition;
  self.client.videoController.captureDevicePosition = SINToggleCaptureDevicePosition(current);
}

- (IBAction)onFullScreenTapped:(id)sender {
  UIView *view = [sender view];
  if ([view sin_isFullscreen]) {
    view.contentMode = UIViewContentModeScaleAspectFit;
    [view sin_disableFullscreen:YES];
  } else {
    view.contentMode = UIViewContentModeScaleAspectFill;
    [view sin_enableFullscreen:YES];
  }
}

- (IBAction)onPauseResumeVideoTapped:(id)sender {
  if (_videoPaused) {
    [self.call resumeVideo];
    _videoPaused = NO;
  } else {
    [self.call pauseVideo];
    _videoPaused = YES;
  }
}

- (void)onDurationTimer:(NSTimer *)unused {
  NSInteger duration = (NSInteger)[[NSDate date] timeIntervalSinceDate:[[self.call details] establishedTime]];
  [self setDuration:duration];
}

#pragma mark - SINCallDelegate

- (void)callDidProgress:(id<SINCall>)call {
  [self setCallStatusText:@"ringing..."];
  [[self.client audioController] startPlayingSoundFile:[self pathForSound:@"ringback.wav"] loop:YES error:nil];
}

- (void)callDidEstablish:(id<SINCall>)call {
  [self startCallDurationTimerWithSelector:@selector(onDurationTimer:)];
  self.remoteVideoView.hidden = NO;
  [self showButtons:kButtonsHangup];
  [[self.client audioController] stopPlayingSoundFile];
}

- (void)callDidEnd:(id<SINCall>)call {
  [self.callKitMediator removeObserver:self];

  [self stopCallDurationTimer];
  [[self.client audioController] stopPlayingSoundFile];
  [[self.client audioController] disableSpeaker];
  [self dismiss];
}

- (void)callDidAddVideoTrack:(id<SINCall>)call {
  [self.remoteVideoView addSubview:[self.client.videoController remoteView]];
  // The callee can answer the call before the video track is available, or after. In makes sense to show
  // remoteVideoView as soon as it's available in the first case only
  self.remoteVideoView.hidden = (call.state != SINCallStateEstablished);
}

- (void)callDidPauseVideoTrack:(id<SINCall>)call {
  self.remoteVideoView.hidden = YES;
}

- (void)callDidResumeVideoTrack:(id<SINCall>)call {
  self.remoteVideoView.hidden = NO;
}

- (void)showRemoteVideoView {
  [self.remoteVideoView addSubview:[self.client.videoController remoteView]];
  self.remoteVideoView.hidden = NO;
}

#pragma mark - Sounds

- (NSString *)pathForSound:(NSString *)soundName {
  return [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:soundName];
}

@end
